@@ -12,6 +12,8 @@
 ----------------------------------------------------------------------------
 module Data.Semigroup.Foldable
   ( Foldable1(..)
+  , intercalate1
+  , intercalateMap1
   , traverse1_
   , for1_
   , sequenceA1_
@@ -74,6 +76,37 @@ instance Foldable1 NonEmpty where
 instance Foldable1 ((,) a) where
   foldMap1 f (_, x) = f x
 
+newtype JoinWith a = JoinWith {joinee :: (a -> a)}
+
+instance Semigroup a => Semigroup (JoinWith a) where
+  JoinWith a <> JoinWith b = JoinWith $ \j -> a j <> j <> b j
+
+-- | Insert an 'm' between each pair of 't m'.  Equivalent to
+-- 'intercalateMap1' with 'id' as the second argument.
+--
+-- >>> intercalate1 ", " $ "hello" :| ["how", "are", "you"]
+-- "hello, how, are, you"
+--
+-- >>> intercalate1 ", " $ "hello" :| []
+-- "hello"
+--
+-- >>> intercalate1 mempty $ "I" :| ["Am", "Fine", "You?"]
+-- "IAmFineYou?"
+intercalate1 :: (Foldable1 t, Semigroup m) => m -> t m -> m
+intercalate1 = flip intercalateMap1 id
+{-# INLINE intercalate1 #-}
+
+-- | Insert 'm' between each pair of 'm' derived from 'a'.
+--
+-- >>> intercalateMap1 " " show $ True :| [False, True]
+-- "True False True"
+--
+-- >>> intercalateMap1 " " show $ True :| []
+-- "True"
+intercalateMap1 :: (Foldable1 t, Semigroup m) => m -> (a -> m) -> t a -> m
+intercalateMap1 j f = flip joinee j . foldMap1 (JoinWith . const . f)
+{-# INLINE intercalateMap1 #-}
+
 newtype Act f a = Act { getAct :: f a }
 
 instance Apply f => Semigroup (Act f a) where
